-- behaviour of the decode stage
-- contains 3 decoders

architecture behaviour of decode is

  component ir_decode_1p
  port (instr_in : in dlx_word;
	dec1_out : out dec_1_states);
  end component;

  component ir_decode_2p
  port (instr_in : in dlx_word;
	dec2_out : out dec_2_states;
	mdr_ctrl_out : out std_logic_vector(0 to 2));
  end component;

  component ir_decode_3
  port (instr_in : in dlx_word;
	s1_sel        : out std_logic;
	rd_s2_adr_sel : out std_logic;
	rd_adr_set31  : out std_logic;
	priv_ins_out  : out std_logic);
  end component;


signal instruction : dlx_word;
signal dec_1_state : dec_1_states;
signal dec_2_state : dec_2_states;

-- decoder output signals
----------------------------------------------

signal s1_sel : std_logic;
signal rd_adr_set31 : std_logic;
signal rd_s2_adr_sel : std_logic;
signal priv_ins : std_logic;

-----------------------------------------------
-- decoded control signals
-----------------------------------------------

-- for execution stage

signal  s1_enab : std_logic_vector(0 to 5);
signal  s2_enab : std_logic_vector(0 to 4);
signal  alu_op_sel : std_logic_vector(0 to 3);
signal  dest_enab : std_logic_vector(0 to 5);
signal  const_sel : std_logic_vector(0 to 1);
signal  immed_sel : std_logic_vector(0 to 1);
signal  exc_enab : std_logic_vector(0 to 7);
signal  icr_shifter : std_logic;
signal  iar_mux : std_logic;
signal  iar_latch : std_logic;

signal  store     : std_logic;
signal  memory_op : std_logic;


-- for memory stage

signal mem_ctrl  : std_logic_vector(0 to 5);
signal entry_mux : std_logic;
signal c_latch   : std_logic;
signal branch  : std_logic;
signal test_and_set : test_and_set_instr;
signal lmdr_latch : std_logic;

-- for write back stage

signal wb_mux : std_logic;
signal rf_in : std_logic;

begin

  --
  -- instruction decoder 1
  -- decodes most instructions except load/store
  --
  decode_1: ir_decode_1p
    port map (instr_in => instruction, dec1_out => dec_1_state);
  --
  -- instruction decoder 2
  -- decodes exact load/store instruction
  --
  decode_2: ir_decode_2p
    port map (instr_in => instruction, dec2_out => dec_2_state,
	      mdr_ctrl_out => mdr_ctrl); 
  --
  -- instruction decoder 3
  -- set signals for the generation of rd, s1(X), s2(Y) 
  --
  decode_3: ir_decode_3
    port map (instr_in => instruction, s1_sel => s1_sel,
	      rd_s2_adr_sel => rd_s2_adr_sel, rd_adr_set31 => rd_adr_set31,
	      priv_ins_out => priv_ins);

-----------------------------------------------------------------------------
decode_input : process(phi1, stall)
 begin
  if phi1 = '1' then
   if stall = '1' then
     instruction <= instr_in;
   else
     instruction <= "00000000000000000000000000000000";
   end if;
  end if;

 end process decode_input;

-- output for the next stage (exec)
-----------------------------------------------------------------------------
 s1_enab_out <= s1_enab;
 s2_enab_out(0) <= s2_enab(0) and
                (not rd_s2_adr_sel or s2_enab(2));      -- |b|
 s2_enab_out(1) <= s2_enab(1) and
                (rd_s2_adr_sel or s2_enab(2));          -- |immed|
 s2_enab_out(2 to 3) <= s2_enab(3 to 4);                -- |pc|constb

 --
 --
 alu_op_sel_out <= alu_op_sel;
 --
 --
 dest_enab_out(0) <= dest_enab(0);
 dest_enab_out(1) <= dest_enab(1) and
                     ((s1_sel nand instruction(20)) or dest_enab(5)); -- |icr|
 dest_enab_out(2) <= dest_enab(2) and
                     ((s1_sel nand instruction(19)) or dest_enab(5)); -- |iar|
 dest_enab_out(3) <= dest_enab(3) and
                     ((s1_sel nand instruction(18)) or dest_enab(5)); -- |tbr|
 dest_enab_out(4) <= dest_enab(4);
 
 --
 --
 const_sel_out <= const_sel;
 immed_sel_out <= immed_sel;
 exc_enab_out <= exc_enab;
 icr_shifter_out <= icr_shifter;
 iar_mux_out <= iar_mux;
 iar_latch_out <= iar_latch;
 smdr_latch_out <= store or memory_op;

-- output for the memory stage
-----------------------------------------------------------------------------
 mem_ctrl_out <= mem_ctrl;
 entry_mux_out <= entry_mux;
 c_latch_out <= c_latch;
 branch_out <= branch;
 test_and_set_out <= test_and_set;
 lmdr_latch_out <= lmdr_latch;

-- output for the write back stage
------------------------------------------------------------------------------
 wb_mux_out <= wb_mux;
 rf_in_out <= rf_in;


  ----------------------------------------------------------------
  -- Shift control signals from decoder 1
  ----------------------------------------------------------------
  output_1 : process (dec_1_state, reg_value)

  begin
    test_and_set <= none;
    branch <= disable;
    case dec_1_state is
      ----------------------------------------------------------------
      when nop_1 =>
        --
        --
        memory_op <= disable after tpd_fsm;
        s1_enab <= s1_none after tpd_fsm;
        s2_enab <= s2_none after tpd_fsm;
        alu_op_sel <= alu_dcare after tpd_fsm;
	dest_enab <= dest_none after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;
        immed_sel <= imm_dcare after tpd_fsm;
        exc_enab <= exc_none after tpd_fsm;
        --
        branch <= disable after tpd_fsm;
      ----------------------------------------------------------------
      when memory =>
	--
	-- MAR  <- A + imm_s16 in exec stage
	-- 
	--
        memory_op <= enable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;       
	s2_enab <= s2_immed after tpd_fsm;
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;     
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;       -- don't care
	immed_sel <= imm_s16 after tpd_fsm;       
	exc_enab <= exc_none after tpd_fsm;           
	--
        rf_in <= 'Z' after tpd_fsm;
        wb_mux <= lmdr after tpd_fsm;
      ----------------------------------------------------------------
      when movi2s =>
	--
	-- move gpr to S : X <- a (X = iar, icr, tbr)
	--
	s1_enab <= s1_a after tpd_fsm;              
	s2_enab <= s2_none after tpd_fsm;        
	alu_op_sel <= alu_pass_s1 after tpd_fsm;     
	dest_enab <= dest_X after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;     -- don't care 
	immed_sel <= imm_dcare after tpd_fsm;       -- don't care
	exc_enab <= exc_none after tpd_fsm;
        iar_mux <= mov after tpd_fsm;    
      ----------------------------------------------------------------
      when movs2i =>
	--
	-- move S to gpr : c <- X (X = iar, icr, tbr)
	--
	s1_enab <= s1_none after tpd_fsm; -- ACHTUNG, HIER NOCH ZIEL EINGEBEN
	s2_enab <= s2_none after tpd_fsm;
	alu_op_sel <= alu_pass_s1 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;     -- don't care
	immed_sel <= imm_dcare after tpd_fsm;       -- don't care
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when br_eqz =>
	--
	-- execute branch : temp(mar) <- pc + imm_s16
	--
	s1_enab <= s1_immed after tpd_fsm;
	s2_enab <= s2_pc after tpd_fsm;
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;         -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
        if reg_value = '0' then
	     branch <= enable after tpd_fsm;
        end if;
      ----------------------------------------------------------------
      when br_nez =>
	--
	-- execute branch : temp(mar) <- pc + imm_s16
	--
	s1_enab <= s1_immed after tpd_fsm;
	s2_enab <= s2_pc after tpd_fsm;
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;         -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
        if reg_value = '1' then
	     branch <= enable after tpd_fsm;
        end if;
      ----------------------------------------------------------------
      when jump =>
	--
	-- calculate jump addr: temp(mar) <- pc + imm_s26
	--
	s1_enab <= s1_immed after tpd_fsm;              
	s2_enab <= s2_pc after tpd_fsm;          
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;        
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;           -- don't care 
	immed_sel <= imm_s26 after tpd_fsm;       
	exc_enab <= exc_none after tpd_fsm;
	--
	branch <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when jump_reg =>
	--
	-- calculate jump addr: temp(mar) <- a 
	--
	s1_enab <= s1_a after tpd_fsm;              
	s2_enab <= s2_none after tpd_fsm;
	alu_op_sel <= alu_pass_s1 after tpd_fsm;
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;          -- don't care
	immed_sel <= imm_dcare after tpd_fsm;            -- don't care
	exc_enab <= exc_none after tpd_fsm;
	--
	branch <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when jump_lnk_1 =>
	--
	-- save pc : r31 <- pc
	--
	s1_enab <= s1_immed after tpd_fsm;              
	s2_enab <= s2_pc after tpd_fsm;                    
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;        
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care 
	immed_sel <= imm_dcare after tpd_fsm;          -- don't care
	exc_enab <= exc_none after tpd_fsm;
        --
	entry_mux <= pc after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	branch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
       ----------------------------------------------------------------
       when jump_lnk_reg_1 => 
	--
	-- calculate jump addr: temp(mar) <- a 
	--
	s1_enab <= s1_a after tpd_fsm;              
	s2_enab <= s2_none after tpd_fsm;               
	alu_op_sel <= alu_pass_s1 after tpd_fsm;        
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;          -- don't care 
	immed_sel <= imm_dcare after tpd_fsm;            -- don't care
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= pc after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	branch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
       ----------------------------------------------------------------
       when trap_1 =>
	--
	-- software interrupt : iar <- pc
	--
	s1_enab <= s1_tbr after tpd_fsm;          
	s2_enab <= s2_immed after tpd_fsm;          
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;       
	dest_enab <= dest_mar after tpd_fsm;    
	const_sel <= const_dcare after tpd_fsm;      -- don't care
	immed_sel <= imm_dcare after tpd_fsm;        -- don't care
	exc_enab <= exc_none after tpd_fsm;
	icr_shifter <= enable after tpd_fsm;
	iar_mux <= trap after tpd_fsm;
	iar_latch <= enable after tpd_fsm;
	--
	branch <= enable after tpd_fsm;
       ----------------------------------------------------------------
       when rfe_1 =>
	--
	-- return from interrupt : shift right logical icr
	--
	s1_enab <= s1_iar after tpd_fsm;
	s2_enab <= s2_const after tpd_fsm;
	alu_op_sel <= alu_pass_s1 after tpd_fsm;
	dest_enab <= dest_mar after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;
	immed_sel <= imm_dcare after tpd_fsm;         -- don't care
	exc_enab <= exc_none after tpd_fsm;
	icr_shifter <= enable after tpd_fsm;
	--
	branch <= enable after tpd_fsm;
       ----------------------------------------------------------------
       when lhi =>
	--
	-- lhi: c <- sll(immed,16) 
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_immed after tpd_fsm;     
	s2_enab <= s2_const after tpd_fsm;     
	alu_op_sel <= alu_sll_s1_s2 after tpd_fsm;       
	dest_enab <= dest_aluout after tpd_fsm;      
	const_sel <= const_10 after tpd_fsm;        
	immed_sel <= imm_dcare after tpd_fsm;       -- don't care      
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
       ----------------------------------------------------------------
       when add_1 =>
	--
	-- add: c <- a + y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_add_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
       when sub_1 =>
	--
	-- sub: c <- a - y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
       when and_1 =>
	--
	-- and: c <- a and y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_and_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_u16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
       when or_1 =>
	--
	-- or: c <- a or y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_or_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_u16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
       when xor_1 =>
	--
	-- xor: c <- a xor y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_xor_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_u16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when sll_1 =>
	--
	-- sll: c <- a sll y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_sll_s1_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
     ----------------------------------------------------------------
      when srl_1 =>
	--
	-- srl: c <- a srl y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_srl_s1_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
     ----------------------------------------------------------------
      when sra_1 =>
	--
	-- sra: c <- a sra y / y : b or immed
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_sra_s1_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	entry_mux <= aluout after tpd_fsm;
	c_latch <= enable after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when seq_1 =>
	--
	-- set if s1 equal s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= seq after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when sne_1 =>
	--
	-- set if s1 not equal s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;     
	s2_enab <= s2_Y after tpd_fsm;     
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;       
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;        
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= sne after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when slt_1 =>
	--
	-- set if s1 lesser then s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= slt after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when sle_1 =>
	--
	-- set if s1 lesser or equal s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= sle after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when sgt_1 =>
	--
	-- set if s1 greater then s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= sgt after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when sge_1 =>
	--
	-- set if s1 greater or equal s2
	-- none <- a sub y / y : b or immed
	--
	s1_enab <= s1_a after tpd_fsm;
	s2_enab <= s2_Y after tpd_fsm;
	alu_op_sel <= alu_s1_sub_s2 after tpd_fsm;
	dest_enab <= dest_aluout after tpd_fsm;
	const_sel <= const_dcare after tpd_fsm;        -- don't care
	immed_sel <= imm_s16 after tpd_fsm;
	exc_enab <= exc_none after tpd_fsm;
	--
	test_and_set <= sge after tpd_fsm;
	--
	wb_mux <= c after tpd_fsm;
	rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when ill_opc =>
	--
	--  illegal opcode: shift left logical icr : icr <- sll(icr,16)
	--  set IOC bit in icr (prior to load from dest_bus)
	--
        memory_op <= disable after tpd_fsm;
	s1_enab <= s1_icr after tpd_fsm;
	s2_enab <= s2_const after tpd_fsm;
	alu_op_sel <= alu_sll_s1_s2 after tpd_fsm;
	dest_enab <= dest_icr after tpd_fsm;
	const_sel <= const_10 after tpd_fsm;
	immed_sel <= imm_dcare after tpd_fsm;         -- don't care
	exc_enab <= exc_ioc after tpd_fsm;
      ----------------------------------------------------------------
      when ill_rraf =>
	--
	--  illegal reg.reg.alu function: shift left logical icr : icr <- sll(icr,16)
	--  set IRRA bit in icr (prior to load from dest_bus)
	--
	s1_enab <= s1_icr after tpd_fsm;
	s2_enab <= s2_const after tpd_fsm;
	alu_op_sel <= alu_sll_s1_s2 after tpd_fsm;
	dest_enab <= dest_icr after tpd_fsm;
	const_sel <= const_10 after tpd_fsm;
	immed_sel <= imm_dcare after tpd_fsm;         -- don't care
	exc_enab <= exc_irra after tpd_fsm;
      ----------------------------------------------------------------
     end case;

end process output_1;


  ----------------------------------------------------------------
  -- Shift control signals from decoder 2
  ----------------------------------------------------------------
  output_2 : process (dec_2_state, dec_1_state)
  begin

   lmdr_latch <= disable;

   case dec_1_state is 
    ------------------------------------------------------------------
    when memory =>
    --
    case dec_2_state is
      ----------------------------------------------------------------
      when load_w_1 =>
	--
	-- lmdr <- mem
	--
	mem_ctrl <= mem_lw after tpd_fsm;
	lmdr_latch <= enable after tpd_fsm;
        --
        rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when load_h_1 =>
	--
	-- lmdr <- mem
	--
	mem_ctrl <= mem_lh after tpd_fsm;    
	lmdr_latch <= enable after tpd_fsm;
        --
        rf_in <= enable after tpd_fsm;
      ----------------------------------------------------------------
      when load_b_1 =>
	--
	-- lmdr <- mem 
	--
	mem_ctrl <= mem_lb after tpd_fsm;    
	lmdr_latch <= enable after tpd_fsm;
        --
        rf_in <= enable after tpd_fsm;
      ---------------------------------------------------------------
      when store_w_1 =>
	--
	-- B -> SMDR in exec phase
	-- SMDR -> Mem[MAR] in memory phase   
	--
	--
	store <= enable after tpd_fsm;
	mem_ctrl <= mem_sw after tpd_fsm;
        --
        rf_in <= disable after tpd_fsm;
     ----------------------------------------------------------------
      when store_h_1 =>
	--
	-- B -> SMDR in exec phase
	-- SMDR -> Mem[MAR] in memory phase
	--
	--
	store <= enable after tpd_fsm;
	mem_ctrl <= mem_sh after tpd_fsm;
        --
        rf_in <= disable after tpd_fsm;
      ----------------------------------------------------------------
      when store_b_1 =>
	--
	-- B -> SMDR in exec phase
	-- SMDR -> Mem[MAR] in memory phase
	--
	--
	store <= enable after tpd_fsm;
	mem_ctrl <= mem_sb after tpd_fsm;
        --
        rf_in <= disable after tpd_fsm;    
      ----------------------------------------------------------------
    end case;
    when others =>
      --
      --
      store <= disable after tpd_fsm;
      mem_ctrl <= mem_none after tpd_fsm;
      --
      rf_in <= 'Z' after tpd_fsm;
    -------------------------------------------------------------------      
  end case;
end process output_2;

----------------------------------------------------------------------      
--
-- generate register file addresses
--
  rs1_out <= instruction(6 to 10);
  rs2_out <= instruction(11 to 15);
  rd_out  <= instruction(16 to 20) when rd_s2_adr_sel = '1' and 
             rd_adr_set31 = '0' 
               else  -- Rtype dest.
         instruction(11 to 15) when rd_s2_adr_sel = '0' and rd_adr_set31 = '0' 
               else  -- Itype dest.
            (others => '1'); -- link dest. 

-- 
--


--
-- control signals
----------------------------------------------------------------------

control_output : process(stall, phi1)

begin

 if stall = enable then

  ir_1_latch <= disable;
  

 else

  if phi1 = '1' then
 
   ir_1_latch <= enable;

  else

   ir_1_latch <= disable;
  
  end if;

 end if;

end process control_output;

end behaviour;







